﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Security.AccessControl;
using System.Security.Principal;
using SolutionManager.Business.UI.Froms;
using SolutionManager.IO;
using SolutionManager.IO.Xml;

namespace SolutionManager.Business.UI
{
	public class SolutionControlModel
	{
		#region Fields

		private SearchDirectory _directory = null;

		#endregion

		#region Constructors

		public SolutionControlModel()
		{
		}

		public SolutionControlModel(SearchDirectory directory)
		{
			_directory = directory;
		}

		#endregion

		#region Properties

		public SearchDirectory SearchDirectory
		{
			get
			{
				return _directory;
			}
		}

		public bool IsInitialized
		{
			get
			{
				return (_directory != null);
			}
		}

		#endregion

		#region Methods

		private List<Solution> GetSolutionsFromFileNames(IDirectorySettings directorySettings)
		{
			List<Solution> results = null;

			if (directorySettings != null)
			{
				try
				{
					SearchOption searchOption = SearchOption.TopDirectoryOnly;

					if (directorySettings.SearchChildren)
					{
						searchOption = SearchOption.AllDirectories;
					}

					string[] fileNames = Directory.GetFiles(directorySettings.Path, "*.sln", searchOption);

					if (fileNames != null)
					{
						int fileNamesCount = fileNames.Length;

						if (fileNamesCount > 0)
						{
							Solution solution = null;
							results = new List<Solution>(fileNamesCount);
							SearchDirectory directory = new SearchDirectory(directorySettings.FriendlyName, directorySettings.Path, directorySettings.ParentTreeNode.Name);
							int solutionCount = 0;

							for (int i = 0; i < fileNamesCount; i++)
							{
								solutionCount++;
								solution = new Solution(fileNames[i], directory);
								results.Add(solution);
							}
						}
					}
				}
				catch (Exception e)
				{
					// TODO: Add logging for Addin
				
					System.Windows.Forms.DialogResult result = System.Windows.Forms.MessageBox.Show(string.Format("{0}\nWould you like to continue?", e.Message), "File System Error", System.Windows.Forms.MessageBoxButtons.OKCancel);

					if (result == System.Windows.Forms.DialogResult.OK)
					{
						results = FindSolutionsFilesRecursive(directorySettings, directorySettings.Path);
					}
				}
			}

			return results;
		}

		private List<Solution> FindSolutionsFilesRecursive(IDirectorySettings directorySettings, string directory)
		{
			List<Solution> results = null;

			if ((!string.IsNullOrEmpty(directory)))
			{
				bool allowAccess = CanAccessDirectory(directory);

				if (allowAccess)
				{
					string[] directories = Directory.GetDirectories(directory);

					if (directories != null)
					{
						int directoryCount = directories.Length;

						if (directoryCount > 0)
						{
							results = new List<Solution>();
							Solution solution = null;

							string[] fileNames = null;
							int fileCount = 0;

							for (int i = 0; i < directoryCount; i++)
							{
								allowAccess = CanAccessDirectory(directories[i]);

								if (allowAccess)
								{
									try
									{
										fileNames = Directory.GetFiles(directories[i]);

										if (fileNames != null)
										{
											fileCount = fileNames.Length;

											for (int f = 0; f < fileCount; f++)
											{
												if (fileNames[f].EndsWith(".sln", StringComparison.OrdinalIgnoreCase))
												{
													solution = new Solution(fileNames[f], directorySettings.Directory);
													results.Add(solution);
												}
											}
										}

										FindSolutionsFilesRecursive(directorySettings, directories[i]);
									}
									catch (Exception e)
									{
										System.Windows.Forms.MessageBox.Show(e.Message);
										// TODO: Add logging.
									}
								}
							}
						}
					}
				}
			}

			return results;
		}

		/// <summary>
		/// Checks if the user has permissions to access the directory.
		/// </summary>
		/// <param name="directory">The directory to check permissions on.</param>
		/// <returns>True if the user searching has permissions to access the directory.</returns>
		private bool CanAccessDirectory(string directory)
		{
			bool allowAccess = true;

			DirectorySecurity dirSecurity = Directory.GetAccessControl(directory);

			AppDomain ad = System.Threading.Thread.GetDomain();
			ad.SetPrincipalPolicy(PrincipalPolicy.WindowsPrincipal);

			NTAccount account = new NTAccount(((WindowsPrincipal)System.Threading.Thread.CurrentPrincipal).Identity.Name.ToString());
			AuthorizationRuleCollection ruleCollection = dirSecurity.GetAccessRules(true, true, account.GetType());

			foreach (System.Security.AccessControl.FileSystemAccessRule rule in ruleCollection)
			{
				if (rule.IdentityReference.Value.Equals(account.Value))
				{
					if (rule.AccessControlType == AccessControlType.Deny)
					{
						allowAccess = false;
						break;
					}
				}
			}

			return allowAccess;
		}

		private bool ContainsDirectory(string searchDirectory)
		{
			bool result = false;

			if (_directory != null)
			{
				int directoryCount = _directory.ChildDirectories.Count;

				for (int i = 0; i < directoryCount; i++)
				{
					if (_directory.ChildDirectories[i].Directory.Equals(searchDirectory, StringComparison.Ordinal))
					{
						result = true;
						break;
					}
				}
			}

			return result;
		}

		private void SaveAddinXmlFile(bool windowOpen)
		{
			string xmlDirectory = string.Format(@"{0}\Visual Studio 2008\Addins\", Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments));

			Addin addinXml = new Addin(xmlDirectory);
			addinXml.SaveAddinSettings(windowOpen);
		}

		private void SaveSolutionXmlFile()
		{
			string xmlDirectory = string.Format(@"{0}\Visual Studio 2008\Addins\", Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments));

			Solutions solutionXml = new Solutions(xmlDirectory);
			solutionXml.SaveXmlFile(_directory.ChildDirectories);
		}

		private bool RenameDirectory(SearchDirectory searchDirectory, string oldName, string newName)
		{
			bool result = false;

			if (searchDirectory != null)
			{
				int directoryCount = searchDirectory.ChildDirectories.Count;
				int childDirectoryCount = 0;

				for (int i = 0; i < directoryCount; i++)
				{
					if (searchDirectory.ChildDirectories[i].Name.Equals(oldName))
					{
						searchDirectory.ChildDirectories[i].Name = newName;

						ChangeDirectoryParentNames(searchDirectory.ChildDirectories[i], newName);

						result = true;
						break;
					}
					else
					{
						childDirectoryCount = searchDirectory.ChildDirectories[i].ChildDirectories.Count;

						if (childDirectoryCount > 0)
						{
							RenameDirectory(searchDirectory.ChildDirectories[i], oldName, newName);
						}
					}
				}
			}

			return result;
		}

		private void ChangeDirectoryParentNames(SearchDirectory searchDirectory, string newName)
		{
			if ((searchDirectory != null) && (!string.IsNullOrEmpty(newName)))
			{
				int directoryCount = searchDirectory.ChildDirectories.Count;

				if (directoryCount > 0)
				{
					for (int i = 0; i < directoryCount; i++)
					{
						searchDirectory.ChildDirectories[i].ParentName = newName;
					}
				}

				int solutionCount = searchDirectory.Solutions.Count;

				if (solutionCount > 0)
				{
					for (int i = 0; i < solutionCount; i++)
					{
						searchDirectory.Solutions[i].Parent.Name = newName;
					}
				}
			}
		}


		public void AddSearchDirectory(SolutionManager.Business.UI.Froms.IDirectorySettings directorySettings)
		{
			SearchDirectory directory = directorySettings.ParentTreeNode.Tag as SearchDirectory;

			if (directory != null)
			{
				if (_directory != null)
				{
					bool containsDirectory = ContainsDirectory(directorySettings.Path);

					if (!containsDirectory)
					{
						List<Solution> solutions = GetSolutionsFromFileNames(directorySettings);

						if ((solutions != null))
						{
							int solutionCount = solutions.Count;

							if (solutionCount > 0)
							{
								directorySettings.Directory.ParentName = directory.Name;
								directorySettings.Directory.Solutions.AddRange(solutions);
								directory.ChildDirectories.Add(directorySettings.Directory);
								SaveSolutionXmlFile();
							}
						}
					}
				}
			}
		}

		public void AddVirtualDirectory(SolutionManager.Business.UI.Froms.IDirectorySettings directorySettings)
		{
			SearchDirectory directory = directorySettings.ParentTreeNode.Tag as SearchDirectory;

			if (directory != null)
			{
				string parentName = "Solution Manager";

				if (directorySettings.ParentTreeNode != null)
				{
					parentName = directorySettings.ParentTreeNode.Text;
				}

				SearchDirectory directoryToAdd = new SearchDirectory(directorySettings.FriendlyName, directorySettings.Path, parentName);
				directory.ChildDirectories.Add(directoryToAdd);
				directorySettings.ParentTreeNode.Tag = directory;
				SaveSolutionXmlFile();
			}
		}

		public void Initialize()
		{
			string xmlDirectory = string.Format(@"{0}\Visual Studio 2008\Addins\", Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments));

			// HACK: needs to change to the SolutionManager.dll directory
			Solutions solutionXmlFile = new Solutions(xmlDirectory);

			if (solutionXmlFile.FileExists)
			{
				if (_directory != null)
				{
					List<SearchDirectory> childDirectories = solutionXmlFile.InitializeSolutionXmlFile();
					_directory.ChildDirectories.AddRange(childDirectories);

					if (_directory.ChildDirectories.Count == 0)
					{
						//AddSearchDirectory(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments));
						SaveSolutionXmlFile();
					}
				}
			}
			else
			{
				//AddSearchDirectory(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments));
				SaveSolutionXmlFile();
			}
		}

		public void RenameDirectory(string oldName, string newName)
		{
			int directoryCount = _directory.ChildDirectories.Count;
			int childDirectoryCount = 0;

			for (int i = 0; i < directoryCount; i++)
			{
				if (_directory.ChildDirectories[i].Name.Equals(oldName))
				{
					_directory.ChildDirectories[i].Name = newName;

					ChangeDirectoryParentNames(_directory.ChildDirectories[i], newName);
					break;
				}
				else
				{
					childDirectoryCount = _directory.ChildDirectories[i].ChildDirectories.Count;

					if (childDirectoryCount > 0)
					{
						// Call recursive method.
						if (RenameDirectory(_directory.ChildDirectories[i], oldName, newName))
						{
							break;
						}
					}
				}
			}

			SaveSolutionXmlFile();
		}

		public void SaveAddinSettings(bool windowOpen)
		{
			SaveAddinXmlFile(windowOpen);
			SaveSolutionXmlFile();
		}

		public void SaveSolutionSettings()
		{
			SaveSolutionXmlFile();
		}

		public void SaveSolutionSettings(SearchDirectory directory)
		{
			_directory = directory;
			SaveSolutionXmlFile();
		}

		#endregion
	}
}