﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Xml;
using System.Xml.Linq;

namespace SolutionManager.IO.Xml
{
	internal sealed class Solutions
	{
		#region Fields

		private const string _SolutionFileName = "Solutions.xml";

		private string _fileName = null;
		private XDocument _xDocument = null;
		

		#endregion

		#region Constructors

		internal Solutions()
			: base()
		{
		}

		internal Solutions(string directory)
			: this()
		{
			_fileName = string.Concat(directory, _SolutionFileName);
		}

		#endregion

		#region Properties

		public bool FileExists
		{
			get
			{
				return File.Exists(_fileName);
			}
		}

		#endregion

		#region Methods

		private void SaveXmlFile()
		{
			if (_xDocument != null)
			{
				try
				{
					_xDocument.Save(_fileName);
				}
				catch (Exception e)
				{
					// TODO: Add logging for Addin
					System.Windows.Forms.MessageBox.Show(e.Message);
				}
			}
		}

		private List<SearchDirectory> ParseXmlFile()
		{
			List<SearchDirectory> directories = null;

			if ((_xDocument != null)) 
			{
				IEnumerable<XElement> directoryElements =
					from item in _xDocument.Descendants("Directory")
					where item.Attribute("parentname").Value == "Solution Manager"
					select item;

				if ((directoryElements != null))
				{
					List<XElement> directoryNodes = directoryElements.ToList();
					int directoryCount = directoryNodes.Count;
					directories = new List<SearchDirectory>(directoryCount);

					if (directoryCount > 0)
					{
						SearchDirectory directory = null;

						for (int i = 0; i < directoryCount; i++)
						{
							directory = ParseChildDirectories(directoryNodes[i]);

							if (directory != null)
							{
								directories.Add(directory);
							}
						}

						directory = null;
					}
				}
			}

			return directories;
		}

		private SearchDirectory ParseChildDirectories(XElement childDirectoryNode)
		{
			SearchDirectory directory = null;

			if (childDirectoryNode != null)
			{
				IEnumerable<XElement> directoryElements =
					from item in childDirectoryNode.Descendants("Directory")
					where item.Attribute("parentname").Value.Equals(childDirectoryNode.Attribute("name").Value)
					select item;

				List<XElement> childElements = directoryElements.ToList<XElement>();

				directory = new SearchDirectory(childDirectoryNode.Attribute("name").Value, childDirectoryNode.Attribute("searchroot").Value, childDirectoryNode.Attribute("parentname").Value);
				int childElementCount = childElements.Count;

				if (childElementCount > 0)
				{
					SearchDirectory childDirectory = null;

					for (int i = 0; i < childElementCount; i++)
					{
						childDirectory = ParseChildDirectories(childElements[i]);

						if (childDirectory != null)
						{
							directory.ChildDirectories.Add(childDirectory);
						}
					}
				}

				IEnumerable<XElement> solutionElements =
					from item in childDirectoryNode.Descendants("Solution")
					where item.Attribute("parentname").Value.Equals(childDirectoryNode.Attribute("name").Value)
					select item;

				List<XElement> solutionNodes = solutionElements.ToList<XElement>();

				int solutionCount = solutionNodes.Count;

				if (solutionCount > 0)
				{
					List<Solution> solutions = null;
					Solution solution = null;
					solutions = new List<Solution>(solutionCount);

					for (int s = 0; s < solutionCount; s++)
					{
						solution = new Solution(solutionNodes[s].Attribute("directory").Value, directory);
						solution.Excluded = Convert.ToBoolean(solutionNodes[s].Attribute("excluded").Value);
						solutions.Add(solution);
						directory.Solutions.Add(solution);
					}
				}
			}

			return directory;
		}

		private XElement CreateSolutionsElement(SearchDirectory searchDirectory)
		{
			XElement solutionsElement = null;

			if (searchDirectory != null)
			{
				solutionsElement = new XElement("Solutions");
				int solutionCount = searchDirectory.Solutions.Count;

				if (solutionCount > 0)
				{
					XElement element = null;
					XAttribute attribute = null;

					for (int i = 0; i < solutionCount; i++)
					{
						element = new XElement("Solution");

						attribute = new XAttribute("name", searchDirectory.Solutions[i].Name);
						element.Add(attribute);

						attribute = new XAttribute("parentname", searchDirectory.Name);
						element.Add(attribute);

						attribute = new XAttribute("excluded", searchDirectory.Solutions[i].Excluded);
						element.Add(attribute);

						attribute = new XAttribute("directory", searchDirectory.Solutions[i].FullName);
						element.Add(attribute);

						attribute = new XAttribute("createDate", searchDirectory.Solutions[i].CreationTime.ToShortDateString());
						element.Add(attribute);

						solutionsElement.Add(element);
					}
				}
			}

			return solutionsElement;
		}

		private XElement AddChildDirectory(SearchDirectory directory)
		{
			XElement directoryElement = null;

			if (directory != null)
			{
				directoryElement = new XElement("Directory");

				XAttribute attribute = new XAttribute("name", directory.Name);
				directoryElement.Add(attribute);

				attribute = new XAttribute("parentname", directory.ParentName);
				directoryElement.Add(attribute);

				attribute = new XAttribute("excluded", directory.Excluded);
				directoryElement.Add(attribute);

				attribute = new XAttribute("searchroot", directory.Directory);
				directoryElement.Add(attribute);

				int directoryCount = directory.ChildDirectories.Count;

				if (directoryCount > 0)
				{
					XElement childDirectoryElement = null;

					for (int i = 0; i < directoryCount; i++)
					{
						childDirectoryElement = AddChildDirectory(directory.ChildDirectories[i]);

						if (childDirectoryElement != null)
						{
							directoryElement.Add(childDirectoryElement);
						}
					}
				}

				int solutionCount = directory.Solutions.Count;

				if (solutionCount > 0)
				{
					XElement solutionElement = CreateSolutionsElement(directory);
					directoryElement.Add(solutionElement);
				}
			}

			return directoryElement;
		}


		public List<SearchDirectory> InitializeSolutionXmlFile()
		{
			List<SearchDirectory> directories = null;

			if (File.Exists(_fileName))
			{
				try
				{
					_xDocument = XDocument.Load(_fileName);
					directories = ParseXmlFile();
				}
				catch (Exception e)
				{
					// TODO: Add logging for Addin.
					System.Windows.Forms.MessageBox.Show(string.Format("Could not load Solutions.xml file.\n{0}", e.Message));
				}
			}

			return directories;
		}

		public void SaveXmlFile(List<SearchDirectory> directories)
		{
			if (directories != null)
			{
				int directoryCount = directories.Count;

				if (directoryCount > 0)
				{
					XElement directoryElement = null;
					XElement parentElement = new XElement("SolutionManager");

					for (int i = 0; i < directoryCount; i++)
					{
						directoryElement = AddChildDirectory(directories[i]);
						parentElement.Add(directoryElement);
					}

					_xDocument = new XDocument(parentElement);

					SaveXmlFile();
				}
			}
		}

		#endregion
	}
}